
body {
	margin:0;
	padding:0;
	/*font-size: 10px;*/
}


/**
 * TOP
 */
#top {
	position: relative;
}
#top .topright {
	position: absolute;
	right: 5px;
	top: 5px;
}

#top .logo {
   /*font-family: cursive;
   color:  #3366CC;*/
   font-size: 30px;
   font-weight: bold;
   padding-left: 10px;
   padding-top: 4px;
}

/* Toolbar */
#toolbar {
   position: absolute;
   top: 10px;
   left: 300px;
}

#toolbar .yui-button {
   margin: 0px;
}
#toolbar .yui-button button {
   padding-left: 2em;
   font-size: 92%;
}

#WiringEditor-saveButton button {
 background:transparent url(../images/icons/database_save.png) no-repeat scroll 10% 50%;

}
#WiringEditor-deleteButton button {
 background:transparent url(../images/icons/delete.png) no-repeat scroll 10% 50%;
}

#WiringEditor-newButton button {
 background:transparent url(../images/icons/add.png) no-repeat scroll 10% 50%;
}

#WiringEditor-loadButton button {
 background:transparent url(../images/icons/application_go.png) no-repeat scroll 10% 50%;
}

#WiringEditor-helpButton button {
 background:transparent url(../images/icons/help.png) no-repeat scroll 10% 50%;
}

/**
 * savedStatus
 */
div.savedStatus {
	color:orange;
	font-size:27pt;
	left:650px;
	position:absolute;
	top:10px;
}

/**
 * LEFT
 */
 
#left {
   
}
 
 
/**
 * Module list 
 */
 
.WiringEditor-module {
   cursor: pointer;
   background-color: #AADDDD;
   margin: 5px;
   padding: 2px;
   font-size: 9pt;
   -moz-border-radius: 2px;
   height: 16px;
   text-align: left;
}

#left .WiringEditor-module:hover {
   color: red;
}

.WiringEditor-module img {
   margin-left: 5px;
}

.WiringEditor-module span {
   margin-left: 5px;
}

#moduleProxy {
   opacity: 0.8;
   filter: alpha(opacity = 80);
}

/**
 * CENTER
 */

#center {
	height: 100%;
}


/**
 * RIGHT
 */
 
#right #propertiesForm {
	font-size: 10px;
}

#layerMap {
   width: 80%;
}

#layerMap canvas.WireIt-LayerMap {
	position: relative;
	/*width: 250px; Don't put any width or it breaks the layerMap */
}

#propertiesForm div.inputEx-Group div.inputEx-label {
	width: 50px;
}


/**
 * Dialogs
 */
#loadPanelBody {
	max-height:300px;
	overflow-y:scroll;
	overflow-x:none;
	text-align:left;
}

#loadPanelBody ul {
   margin-left: 20px;
}

#loadPanelBody ul li {
	background-color:#FFFFFF;
	list-style-type:none;
	margin:2px;
	padding:2px 10px;
	-moz-border-radius:7px;
	-webkit-border-radius: 7px;
}

#loadPanelBody ul li:hover {
	background-color: #CCCCFF;
}

#alertPanelBody {
	padding: 10px 0;
}