/* 
Distributed under the MIT License :
Visit http://javascript.neyric.com/inputex for more informations

Copyright (c) 2007-2008, Eric Abouaf <neyric at via.ecp.fr>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
div.inputEx-fieldWrapper{position:relative;margin-bottom:.2em;}div.inputEx-label{float:left;color:#505050;text-align:right;padding:.2em .5em 0 0;}div.inputEx-Group div.inputEx-label{width:130px;}div.inputEx-Field{float:left;position:relative;}div.inputEx-description{clear:both;color:#505050;font-size:75%;margin-top:2px;}div.inputEx-message{float:left;margin:4px;font-size:85%;}div.inputEx-invalid>div.inputEx-message{color:#C00E0C;font-weight:bold;}div.inputEx-invalid>div.inputEx-label{color:#C00E0C;font-weight:bold;}div.inputEx-invalid>div.inputEx-label label{background:transparent url(../images/error_bang.gif) no-repeat scroll 0 1px;padding-left:20px;}div.inputEx-valid>div.inputEx-message{color:#505050;font-weight:normal;}div.inputEx-valid>div.inputEx-label{color:#505050;font-weight:normal;}div.inputEx-valid>div.inputEx-label label,div.inputEx-empty>div.inputEx-label label{background:none;padding-left:inherit;font-weight:normal;color:#505050;}div.inputEx-typeInvite>div.inputEx-Field input,div.inputEx-typeInvite>div.inputEx-Field textarea{color:#aaa;}div.inputEx-empty div.inputEx-valid div.inputEx-Field input,div.inputEx-empty div.inputEx-valid div.inputEx-Field textarea{color:black;}inputEx-focused div.inputEx-Field input,inputEx-focused div.inputEx-Field textarea{color:black;}div.inputEx-StringField-wrapper{float:none;}legend.inputEx-Group-legend{color:#63126B;font-weight:bold;}div.inputEx-Group div.inputEx-Group-collapseImg{width:14px;height:14px;background-image:url(../images/sprite-menu.gif);background-color:transparent;background-repeat:no-repeat;background-attachment:scroll;float:left;}div.inputEx-Group fieldset.inputEx-Expanded legend,div.inputEx-Group fieldset.inputEx-Collapsed legend{cursor:pointer;}div.inputEx-Group fieldset.inputEx-Expanded legend div.inputEx-Group-collapseImg{background-position:-8px 2px;}div.inputEx-Group fieldset.inputEx-Expanded legend:hover div.inputEx-Group-collapseImg{background-position:-8px -78px;}div.inputEx-Group fieldset.inputEx-Collapsed legend div.inputEx-Group-collapseImg{background-position:-8px -317px;}div.inputEx-Group fieldset.inputEx-Collapsed legend:hover div.inputEx-Group-collapseImg{background-position:-8px -397px;}div.inputEx-Group fieldset.inputEx-Collapsed div.inputEx-fieldWrapper{display:none;}label.inputExForm-checkbox-rightLabel{margin-left:4px;}div.inputEx-Form-buttonBar{margin:3px 0 0 137px;}div.inputEx-Form-Mask{height:275px;left:0;position:absolute;top:0;width:180px;}div.inputEx-Form-Mask div.inputEx-Form-Mask-bg{background-color:#FFF;height:100%;left:0;opacity:.9;position:absolute;top:0;width:100%;}div.inputEx-Form-Mask div.inputEx-Form-Mask-spinner{width:36px;height:36px;background-image:url(../images/spinner.gif);}div.inputEx-Form-Mask span{color:#666;}div.inputEx-UrlField input{padding-left:20px;}div.inputEx-UrlField input.nofavicon{padding-left:0;}div.inputEx-UrlField img{position:absolute;top:3px;left:2px;width:16px;height:16px;}img.inputEx-ListField-addButton{background:transparent url(../images/add.png) no-repeat scroll 0;width:16px;height:16px;cursor:pointer;}img.inputEx-ListField-delButton{background:transparent url(../images/delete.png) no-repeat scroll 0;width:16px;height:16px;float:left;cursor:pointer;}div.inputEx-ListField-Arrow{width:16px;height:16px;float:left;cursor:pointer;}div.inputEx-ListField-ArrowUp{background:transparent url(../images/bullet_arrow_up.png) no-repeat scroll 0;}div.inputEx-ListField-ArrowDown{background:transparent url(../images/bullet_arrow_down.png) no-repeat scroll 0;}a.inputEx-List-link{color:blue;cursor:pointer;margin-left:10px;}a.inputEx-List-link:hover{text-decoration:underline;}div.inputEx-InPlaceEdit-visu{cursor:pointer;padding:3px;}div.inputEx-InPlaceEdit-visu:hover{background-color:#ff9;}input.inputEx-InPlaceEdit-OkButton{margin-left:5px;}a.inputEx-InPlaceEdit-CancelLink{margin-left:5px;margin-top:2px;}.yui-skin-sam div.inputEx-AutoComplete .yui-ac-input{width:auto;position:relative;}.yui-skin-sam div.inputEx-AutoComplete .yui-ac-content{width:auto;}div.inputEx-AutoComplete div.yui-ac-container{margin-bottom:2em;width:25em;}div.inputEx-DateField{background:transparent url(../images/calendar.gif) no-repeat scroll left center;padding-left:20px;}div.inputEx-DateField input{width:75px;}div.inputEx-PickerField,div.inputEx-PickerField input{vertical-align:top;}div.inputEx-PickerField{background:none;padding-left:inherit;}div.inputEx-PickerField .yui-menu-button button{background-position:center center;background-repeat:no-repeat;overflow:hidden;font-size:9px;padding:2px;width:20px;height:16px;}div.inputEx-DatePickerField .yui-menu-button button{background-image:url(../images/calendar.gif);}div.inputEx-DatePickerField div.yui-overlay{width:210px;}div.inputEx-ColorField .yui-menu-button button{background-image:url(../images/palette.png);}div.inputEx-ColorField-button{height:18px;width:100px;border:1px solid #B5B8C8;margin-left:0;cursor:pointer;float:left;}div.inputEx-ColorField-square{float:left;cursor:pointer;}div.inputEx-ColorField div.yui-overlay div.bd{background:white no-repeat scroll 5px 5px;border:1px solid #AAA;}div.inputEx-SliderField div.inputEx-SliderField-bg{position:relative;background:url('../images/slider-bg-h.gif') 5px 0 no-repeat;height:28px;width:112px;float:left;}div.inputEx-SliderField div.inputEx-SliderField-thumb{background:url('../images/thumb-n.gif') 0 0 no-repeat;cursor:pointer;position:absolute;top:4px;width:17px;height:21px;}div.inputEx-SliderField div.inputEx-SliderField-value{float:left;margin-top:6px;margin-left:5px;}div.inputEx-PasswordField input{float:left;}div.inputEx-PasswordField div.capsLockWarning{float:left;margin:3px 4px;color:#505050;font-size:85%;}div.inputEx-Password-StrengthIndicator{float:left;color:#505050;font-size:70%;margin-left:10px;width:83px;}div.inputEx-Password-StrengthIndicator div.inputEx-Password-StrengthIndicatorBlock{background-color:white;width:1.4em;height:7px;border:1px solid #aaa;float:left;margin-left:2px;}div.inputEx-CheckBox input{float:left;margin:2px 0 4px 0;}label.inputEx-CheckBox-rightLabel{margin:3px;}div.inputEx-CombineField-separator{float:left;margin-top:2px;}.inputEx-DDList-item{padding:5px;background-color:#DDD;cursor:move;border:1px solid #BBB;width:300px;position:relative;}.inputEx-DDList-item a{color:#D05D00;padding:3px 5px;position:absolute;right:0;top:0;cursor:pointer;}.inputEx-DDList-item a:hover{text-decoration:underline;}label.inputEx-RadioField-rightLabel{cursor:pointer;margin:3px;}div.inputEx-RadioField-choice{float:left;margin-left:10px;}div.inputEx-DataTable-formContainer{float:left;}div.inputEx-DataTable-formContainer div.inputEx-DataTable-formWrapper{position:absolute;width:300px;background-color:white;-moz-border-radius-bottomright:10px;-moz-border-radius-topright:10px;-webkit-border-bottom-right-radius:10px;-webkit-border-top-right-radius:10px;border:1px solid #7F7F7F;padding:10px;}div.inputEx-TypeField-EditButton{width:16px;height:16px;background-image:url(../images/pencil.png);cursor:pointer;float:left;margin-left:4px;position:relative;}div.inputEx-TypeField-EditButton.opened{background-image:url(../images/pencil_delete.png);}div.inputEx-TypeField-FieldValueWrapper{float:left;margin-left:4px;}div.inputEx-TypeField-PropertiesPanel{border:1px solid #aaa;padding:3px;background-color:white;position:absolute;top:0;left:16px;z-index:1;width:380px;}div.inputEx-MenuField div.inputEx-Result{background-color:#FFF;border:1px solid #CCC;padding:2px 25px 2px 5px;cursor:pointer;}div.inputEx-MenuField div.inputEx-RightArrow{background-image:url(../images/menu-right-arrow.png);background-position:right center;background-repeat:no-repeat;}div.inputEx-MenuField div.inputEx-DownArrow{background-image:url(../images/menu-down-arrow.png);background-position:right center;background-repeat:no-repeat;}ul.inputEx-JsonTreeInspector li{padding-left:20px;}ul.inputEx-JsonTreeInspector li.collapsed{background:transparent url(../images/plus.gif) no-repeat scroll 0 0;cursor:pointer;}ul.inputEx-JsonTreeInspector li.expanded{background:transparent url(../images/minus.gif) no-repeat scroll 0 0;cursor:pointer;}ul.inputEx-JsonTreeInspector li span{padding-left:20px;color:green;}ul.inputEx-JsonTreeInspector li span.type-number,ul.inputEx-JsonTreeInspector li span.type-boolean{color:blue;}ul.inputEx-JsonTreeInspector li span.type-null{color:red;}