div.WireIt-Container {
width: 360px; /* Prevent the modules from scratching on the right */
}

div.WireIt-InputExTerminal {
width: 21px;
height: 21px;
position: absolute;
}

div.WireIt-InputExTerminal.output_term {
right:0;
}

div.WireIt-InputExTerminal div.WireIt-Terminal {
top: -7px;
left: -7px;
}

div.WireIt-InputExTerminal.Float  div.WireIt-Terminal {background-image: url(terminals_float.png);}
div.WireIt-InputExTerminal.Int    div.WireIt-Terminal {background-image: url(terminals_int.png);}
div.WireIt-InputExTerminal.String div.WireIt-Terminal {background-image: url(terminals_string.png);}
div.WireIt-InputExTerminal.File   div.WireIt-Terminal {background-image: url(terminals_file.png);}

div.inputEx-Group div.inputEx-label {width: 60px; margin-left: 20px;}

div.WireIt-ImageContainer {width: auto;}

div.Bubble div.body {width: 70px; height: 45px; opacity: 0.8; cursor: move;}

#wireiteditor.readonly div.WireIt-Container-closebutton {display: none}
#wireiteditor label {font-weight: normal;} /* wireit does not use default cw for font weight */

#wireiteditor #left {padding: 7px;background-color: #fff;}
#wireiteditor #left div.section {margin-top: 2em;}
#wireiteditor #left .fieldname {margin-right: 1em; width: 6em; font-weight: bold; display: block; float: left}
#wireiteditor #left div.iformTitle {
  background: #EFEFEF;
  border: 1px solid #EFEFEF;
  font-size: 110%;
  font-weight: bold;
}
#wireiteditor #left div.iformTitle span {margin: 0.5em;}

#wireiteditor div.inputEx-Field input {width: 223px}
#wireiteditor div.inputEx-label label {display: block; overflow: hidden;}

/* do not display invalid symbol on outputs and wired field : */
#wireiteditor div.inputEx-invalid.Wired > div.inputEx-label label,
#wireiteditor div.inputEx-invalid.output > div.inputEx-label label {
  background: none; padding-left: 0; color: #505050;
}

#WiringEditor-alertPanel {text-align: center;}

#wireiteditor.yui-layout .yui-layout-unit div.yui-layout-bd {background-color: #fff;}

#contentmain #propertiesForm div.inputEx-Group div.inputEx-label {width: 90px}
#contentmain #propertiesForm div.inputEx-Group div.inputEx-Field textarea {width: 170px}

